==> v3.32.tar.gz: wcs/wcs/qommon/templatetags/qommon.py <==
# w.c.s. - web application for online forms
# Copyright (C) 2005-2017  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import datetime
from decimal import Decimal

from django import template
from django.template import defaultfilters
from django.utils import dateparse
from django.utils.safestring import mark_safe
from wcs.qommon import evalutils
from wcs.qommon import tokens
from wcs.qommon.admin.texts import TextsDirectory

register = template.Library()

@register.filter
def get(mapping, key):
    return mapping.get(key)

@register.filter
def startswith(string, substring):
    return string and string.startswith(substring)

@register.filter
def split(string, separator=' '):
    return (string or '').split(separator)

@register.filter
def parse_date(date_string):
    try:
        return evalutils.make_date(date_string)
    except ValueError:
        pass
    # fallback to Django function
    try:
        return dateparse.parse_date(date_string)
    except (ValueError, TypeError):
        return None

@register.filter
def parse_datetime(datetime_string):
    try:
        return evalutils.make_datetime(datetime_string)
    except ValueError:
        pass
    # fallback to Django function
    try:
        return dateparse.parse_datetime(datetime_string)
    except (ValueError, TypeError):
        return None

@register.filter
def parse_time(time_string):
    try:
        return dateparse.parse_time(time_string)
    except (ValueError, TypeError):
        return None

@register.filter(expects_localtime=True, is_safe=False)
def date(value, arg=None):
    if not isinstance(value, (datetime.datetime, datetime.date, datetime.time)):
        value = parse_date(value)
    return defaultfilters.date(value, arg=arg)

@register.filter(is_safe=True)
def decimal(value):
    if not value:
        return ''
    try:
        if isinstance(value, basestring):
            # replace , by . for French users comfort
            value = value.replace(',', '.')
        return Decimal(value).quantize(Decimal('1.0000')).normalize()
    except ArithmeticError:
        return ''

@register.simple_tag
def standard_text(text_id):
    return mark_safe(TextsDirectory.get_html_text(str(text_id)))

@register.simple_tag(takes_context=True)
def action_button(context, action_id, label, delay=3):
    from wcs.formdef import FormDef
    formdata_id = context.get('form_number_raw')
    formdef_urlname = context.get('form_slug')
    if not (formdef_urlname and formdata_id):
        return ''
    formdef = FormDef.get_by_urlname(formdef_urlname)
    formdata = formdef.data_class().get(formdata_id, ignore_errors=True)
    token = tokens.Token(expiration_delay=delay*86400, size=64)
    token.type = 'action'
    token.context = {
        'form_slug': formdef_urlname,
        'form_number_raw': formdata_id,
        'action_id': action_id,
        'label': label,
    }
    token.store()
    return '---===BUTTON:%s:%s===---' % (token.id, label)

==> v3.41.tar.gz: wcs/wcs/qommon/templatetags/qommon.py <==
# w.c.s. - web application for online forms
# Copyright (C) 2005-2017  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import datetime
from decimal import Decimal

from django import template
from django.template import defaultfilters
from django.utils import dateparse
from django.utils.safestring import mark_safe
from wcs.qommon import evalutils
from wcs.qommon import tokens
from wcs.qommon.admin.texts import TextsDirectory

register = template.Library()

@register.filter
def get(mapping, key):
    return mapping.get(key)

@register.filter
def startswith(string, substring):
    return string and string.startswith(substring)

@register.filter
def split(string, separator=' '):
    return (string or '').split(separator)

@register.filter
def parse_date(date_string):
    try:
        return evalutils.make_date(date_string)
    except ValueError:
        pass
    # fallback to Django function
    try:
        return dateparse.parse_date(date_string)
    except (ValueError, TypeError):
        return None

@register.filter(expects_localtime=True, is_safe=False)
def date(value, arg=None):
    if arg is None:
        return parse_date(value) or ''
    if not isinstance(value, (datetime.datetime, datetime.date, datetime.time)):
        value = parse_datetime(value) or parse_date(value)
    return defaultfilters.date(value, arg=arg)

@register.filter
def parse_datetime(datetime_string):
    try:
        return evalutils.make_datetime(datetime_string)
    except ValueError:
        pass
    # fallback to Django function
    try:
        return dateparse.parse_datetime(datetime_string)
    except (ValueError, TypeError):
        return None

@register.filter(name='datetime', expects_localtime=True, is_safe=False)
def datetime_(value, arg=None):
    if arg is None:
        return parse_datetime(value) or ''
    if not isinstance(value, (datetime.datetime, datetime.date, datetime.time)):
        value = parse_datetime(value)
    return defaultfilters.date(value, arg=arg)

@register.filter
def parse_time(time_string):
    # if input is a datetime, extract its time
    try:
        dt = parse_datetime(time_string)
        if dt:
            return dt.time()
    except (ValueError, TypeError):
        pass
    # fallback to Django function
    try:
        return dateparse.parse_time(time_string)
    except (ValueError, TypeError):
        return None

@register.filter(expects_localtime=True, is_safe=False)
def time(value, arg=None):
    if arg is None:
        parsed = parse_time(value)
        return parsed if parsed is not None else ''  # because bool(midnight) == False
    if not isinstance(value, (datetime.datetime, datetime.date, datetime.time)):
        value = parse_time(value)
    return defaultfilters.date(value, arg=arg)

def parse_decimal(value):
    if not value:
        return None
    if isinstance(value, basestring):
        # replace , by . for French users comfort
        value = value.replace(',', '.')
    try:
        return Decimal(value).quantize(Decimal('1.0000')).normalize()
    except (ArithmeticError, TypeError):
        return None

@register.filter(is_safe=True)
def decimal(value, arg=None):
    if arg is None:
        return parse_decimal(value) or ''
    if not isinstance(value, Decimal):
        value = parse_decimal(value)
    return defaultfilters.floatformat(value, arg=arg)

@register.simple_tag
def standard_text(text_id):
    return mark_safe(TextsDirectory.get_html_text(str(text_id)))

@register.simple_tag(takes_context=True)
def action_button(context, action_id, label, delay=3):
    from wcs.formdef import FormDef
    formdata_id = context.get('form_number_raw')
    formdef_urlname = context.get('form_slug')
    if not (formdef_urlname and formdata_id):
        return ''
    formdef = FormDef.get_by_urlname(formdef_urlname)
    formdata = formdef.data_class().get(formdata_id, ignore_errors=True)
    token = tokens.Token(expiration_delay=delay*86400, size=64)
    token.type = 'action'
    token.context = {
        'form_slug': formdef_urlname,
        'form_number_raw': formdata_id,
        'action_id': action_id,
        'label': label,
    }
    token.store()
    return '---===BUTTON:%s:%s===---' % (token.id, label)
